/* Copyright 1996, Jan D. Wolter and Steven R. Weiss, All Rights Reserved. */

#include "backtalk.h"

#include <time.h>

#include "dict.h"
#include "print.h"
#include "sysdict.h"


/* PRINT_TOKEN - Print the a token, in input format so we can see its type.
 */

void print_token(FILE *fp, Token *t)
{

    if (type(*t) == TK_INTEGER) 
	fprintf(fp,"%d",ival(*t));

    else if (type(*t) == TK_TIME) 
    {
	/* There isn't really an input form for times, so fudge it */
	char *tm= ctime((time_t *)&t->val);
	tm[7]= tm[10]= tm[19]= '_';
	tm[24]= '\0';
	fputs(tm+4,fp);
    }

    else if (type(*t) == TK_UNBOUND_LITERAL)
	fprintf(fp,"/%s",sval(*t));
    else if (type(*t) == TK_BOUND_LITERAL) 
	fprintf(fp,"/%s",sysdict[ival(*t)].label);

    else if (type(*t) == TK_UNBOUND_SYMBOL)
	fputs(sval(*t),fp);
    else if (type(*t) == TK_BOUND_SYMBOL) 
	fputs(sysdict[ival(*t)].label,fp);

    else if (type(*t) == TK_MARK)
	fprintf(fp,"MARK[%d]",ival(*t));

    else if (type(*t) == TK_FUNCTION)
	fputs(function_name(t->val),fp);

    else if (type(*t) == TK_REGEX)
	fputs("<<regex>>",fp);

    else if (type(*t) == TK_STRING)
    {
	char *cp;
	fputc('(',fp);
	for(cp= sval(*t); *cp != '\0'; cp++)
	    switch (*cp)
	    {
	    case '\n': fputs("\\n",fp); break;
	    case '\b': fputs("\\b",fp); break;
	    case '\t': fputs("\\t",fp); break;
	    case '\\': fputs("\\\\",fp); break;
	    case '(':  fputs("\\(",fp); break;
	    case ')':  fputs("\\)",fp); break;
	    case '<':  fputs(ttymode ? "<" : "&lt;",fp); break;
	    case '>':  fputs(ttymode ? ">" : "&gt;",fp); break;
	    case '&':  fputs(ttymode ? "&" : "&amp;",fp); break;
	    default:   fputc(*cp,fp); break;
	    }
	fputc(')',fp);
    }
    else if (type(*t) == TK_PROCEDURE || type(*t) == TK_ARRAY)
    {
	int i;
	fputc((type(*t) == TK_ARRAY) ? '[' : '{' ,fp);
	for (i= 0; i < aval(*t)->sz; i++)
	{
	    if (i != 0) fputc(' ',fp);
	    print_token(fp,&(aval(*t)->a[i]));
	}
	fputc((type(*t) == TK_ARRAY) ? ']' : '}' ,fp);
    }
}


/* PRINT_VALUE - Print the value of a token, without giving any clue to its
 * type.  The contents of arrays and procedures are printed recursively.
 * Marks and regular expressions are not printed at all.
 */

void print_value(FILE *fp, Token *t)
{
    if (type(*t) == TK_INTEGER) 
	fprintf(fp,"%d",ival(*t));
    else if (type(*t) == TK_TIME)
    {
	char *tm= ctime((time_t *)&t->val);
	tm[24]= '\0';
	fputs(tm,fp);
    }
    else if (type(*t) == TK_BOUND_SYMBOL || type(*t) == TK_BOUND_LITERAL) 
	fputs(sysdict[ival(*t)].label,fp);
    else if (class(*t) == TKC_STRING)
	fputs(sval(*t),fp);
    else if (class(*t) == TKC_ARRAY)
    {
	int i;
	for (i= 0; i < aval(*t)->sz; i++)
	    print_value(fp,&(aval(*t)->a[i]));
    }
}


/* FUNCTION_NAME - Return the name of a built-in function.
 */

char *function_name(void *func)
{
int i;

    for (i= 0; i < SYSDICTLEN; i++)
	if (sysdict[i].t.val == func && sysdict[i].t.flag & TK_FUNCTION)
	    return sysdict[i].label;
    return "(function)";
}
